TARGET_SCORE: int = 0.7
END_INDEX_POSITION: int = 4
START_INDEX_POSITION: int = 3
SCORE_THRESHOLD = 0.7

TOPONYM_PATTERN = r"путепровод|улица|набережная реки|проспект"\
            r"|бульвар|мост|переулок|площадь|переулок"\
            r"|набережная|канала|канал|дорога на|дорога в"\
            r"|шоссе|аллея|проезд|линия"

TARGET_TOPONYMS = [
    "пр",
    "проспект",
    "проспекте",
    "ул",
    "улица",
    "улице",
    "площадь",
    "площади",
    "пер",
    "переулок",
    "проезд",
    "проезде",
    "дорога",
    "дороге",
    "линия",
    "линии"
]
REPLACEMENT_DICT = {
    "пр": "проспект",
    "ул": "улица",
    "пер": "переулок",
    "улице": "улица",
    "проспекте": "проспект",
    "площади": "площадь",
    "проезде": "проезд",
    "дороге": "дорога",
    "линии": "линия"
}

GLOBAL_CRS = 4326
GLOBAL_METRIC_CRS = 3857

OSM_TAGS = {
    "subway": ["yes"],
    "amenity": ["university", "school"],
    "landuse": [
        "brownfield",
        "cemetery",
        "commercial",
        "construction",
        "flowerbed",
        "grass",
        "industrial",
        "meadow",
        "military",
        "plant_nursery",
        "recreation_ground",
        "religious",
        "residential",
        "retail",
    ],
    "natural": ["water", "beach"],
    "leisure": [
        "garden",
        "marina",
        "nature_reserve",
        "park",
        "pitch",
        "sports_centre",
    ],
    "highway": [
        "construction",
        "footway",
        "motorway",
        "pedestrian",
        "primary",
        "primary_link",
        "residential",
        "secondary",
        "service",
        "steps",
        "tertiary",
        "tertiary_link",
        "unclassified",
    ],
    "railway": ["rail", "subway"],
    "amenity": [
        "arts_centre",
        "atm",
        "bank",
        "bar",
        "boat_rental",
        "bus_station",
        "bicycle_rental",
        "biergarten",
        "cafe",
        "car_wash",
        "childcare",
        "cinema",
        "clinic",
        "clinic;doctors;audiologist",
        "college",
        "community_centre",
        "courthouse",
        "coworking_space",
        "dancing_school",
        "dentist",
        "doctors",
        "driving_school",
        "events_venue",
        "fast_food",
        "fire_station",
        "food_court",
        "fountain",
        "fuel",
        "hookah_lounge",
        "hospital",
        "internet_cafe",
        "kindergarten",
        "language_school",
        "library",
        "music_school",
        "music_venue",
        "nightclub",
        "offices",
        "parcel_locker",
        "parking",
        "payment_centre",
        "pharmacy",
        "place_of_worship",
        "police",
        "post_office",
        "pub",
        "recycling",
        "rescue_station",
        "restaurant",
        "school",
        "social_centre",
        "social_facility",
        "studio",
        "theatre",
        "training",
        "university",
        "vending_machine",
        "veterinary",
        "townhall",
        "shelter",
        "marketplace",
        "monastery",
        "planetarium",
        "research_institute",
    ],
    "building": [
        "apartments",
        "boat",
        "bunker",
        "castle",
        "cathedral",
        "chapel",
        "church",
        "civic",
        "college",
        "commercial",
        "detached",
        "dormitory",
        "garages",
        "government",
        "greenhouse",
        "hospital",
        "hotel",
        "house",
        "industrial",
        "kindergarten",
        "kiosk",
        "mosque",
        "office",
        "pavilion",
        "policlinic",
        "public",
        "residential",
        "retail",
        "roof",
        "ruins",
        "school",
        "service",
        "ship",
        "sport_centre",
        "sports_hall",
        "theatre",
        "university",
    ],
    "man_made": [
        "bridge",
        "courtyard",
        "lighthouse",
        "mineshaft",
        "pier",
        "satellite",
        "tower",
        "works",
    ],
    "leisure": [
        "amusement_arcade",
        "fitness_centre",
        "playground",
        "sauna",
        "stadium",
        "track",
    ],
    "office": [
        "company",
        "diplomatic",
        "energy_supplier",
        "government",
        "research",
        "telecommunication",
    ],
    "shop": [
        "alcohol",
        "antiques",
        "appliance",
        "art",
        "baby_goods",
        "bag",
        "bakery",
        "bathroom_furnishing",
        "beauty",
        "beauty;hairdresser;massage;cosmetics;perfumery",
        "bed",
        "beverages",
        "bicycle",
        "binding",
        "bookmaker",
        "books",
        "boutique",
        "butcher",
        "car",
        "car_parts",
        "car_repair",
        "carpet",
        "cheese",
        "chemist",
        "clothes",
        "coffee",
        "computer",
        "confectionery",
        "convenience",
        "copyshop",
        "cosmetics",
        "cosmetics;chemist",
        "craft",
        "craft;paint",
        "curtain",
        "dairy",
        "deli",
        "doityourself",
        "doors",
        "dry_cleaning",
        "e-cigarette",
        "electrical",
        "electronics",
        "electronics;fishing",
        "erotic",
        "estate_agent",
        "fabric",
        "farm",
        "fireplace",
        "fishing",
        "flooring",
        "florist",
        "frame",
        "frozen_food",
        "furniture",
        "games",
        "garden_centre",
        "gas",
        "general",
        "gift",
        "glaziery",
        "gold_buyer",
        "greengrocer",
        "hairdresser",
        "hairdresser_supply",
        "hardware",
        "health_food",
        "hearing_aids",
        "herbalist",
        "honey",
        "houseware",
        "interior_decoration",
        "jeweller_tools",
        "jewelry",
        "kiosk",
        "kitchen",
        "laundry",
        "leather",
        "lighting",
        "lottery",
        "massage",
        "medical_supply",
        "mobile_phone",
        "money_lender",
        "motorcycle",
        "music",
        "newsagent",
        "nuts",
        "optician",
        "orthopaedic",
        "orthopaedics",
        "outdoor",
        "outpost",
        "paint",
        "pastry",
        "pawnbroker",
        "perfumery",
        "pet",
        "pet_grooming",
        "photo",
        "pottery",
        "printer_ink",
        "printing",
        "radiotechnics",
        "repair",
        "seafood",
        "second_hand",
        "security",
        "sewing",
        "shoes",
        "sports",
        "stationery",
        "stationery;copyshop",
        "storage_rental",
        "supermarket",
        "tableware",
        "tailor",
        "tattoo",
        "tea",
        "ticket",
        "tobacco",
        "toys",
        "travel_agency",
        "variety_store",
        "watches",
        "water_filter",
        "weapons",
        "wine",
    ],
    "bus": ["yes"],
    "public_transport": ["platform", "station", "stop_position"],
    "railway": ["tram_stop", "station"],
}

TAG_ROUTER = {"NOUN": "содержит", "ADJF": "описание", "ADJS": "описание", "VERB": "активность", "INFN": "активность"}

STOPWORDS = [
    "фото",
    "улица",
    "улицы",
    "улице",
    "улицу",
    "улицей",
    "улицею",
    "улиц",
    "улицам",
    "улицами",
    "улицах",
    "дом",
    "дома",
    "дому",
    "домом",
    "доме",
    "домов",
    "домам",
    "домами",
    "домах",
    "проспект",
    "проспекта",
    "проспекту",
    "проспектом",
    "проспекте",
    "проспекты",
    "проспектов",
    "проспектам",
    "проспектами",
    "проспектах",
    "дорога",
    "дороги",
    "дороге",
    "дорогу",
    "дорогой",
    "дорогою",
    "дорог",
    "дорогам",
    "дорогами",
    "дорогах",
    "час",
    "часа",
    "часу",
    "часом",
    "часе",
    "часы",
    "часов",
    "часам",
    "часами",
    "часах",
    "год",
    "года",
    "году",
    "годом",
    "годе",
    "годы",
    "лета",
    "годов",
    "лет",
    "годам",
    "годами",
    "годах",
    "гг",
    "летам",
    "летами",
    "летах",
    "утро",
    "утра",
    "утру",
    "утром",
    "утре",
    "утр",
    "утрам",
    "утрами",
    "утрах",
    "здравствуйте",
    "ул",
    "пр",
    "здание",
    "зданье",
    "здания",
    "зданья",
    "зданию",
    "зданью",
    "зданием",
    "зданьем",
    "здании",
    "зданьи",
    "зданий",
    "зданиям",
    "зданьям",
    "зданиями",
    "зданьями",
    "зданиях",
    "зданьях",
    "город",
    "города",
    "городу",
    "городом",
    "городе",
    "городов",
    "городам",
    "городами",
    "городах",
    "аноним",
    "анонима",
    "анониму",
    "анонимом",
    "анониме",
    "анонимы",
    "анонимов",
    "анонимам",
    "анонимами",
    "анонимах",
    "день",
    "дня",
    "дню",
    "днём",
    "дне",
    "дни",
    "дней",
    "дням",
    "днями",
    "днях",
    "вечер",
    "вечера",
    "вечеру",
    "вечером",
    "вечере",
    "вечеров",
    "вечерам",
    "вечерами",
    "вечерах" "адрес",
    "адреса",
    "адресу",
    "адресом",
    "адресе",
    "адресы",
    "адресов",
    "адресам",
    "адресами",
    "адресах",
]
SPB_DISTRICTS = [
    {"name": "Санкт-Петербург", "admin_level": 4, "parent": None},
    {"name": "Адмиралтейский", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Василеостровский", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Выборгский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Калининский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Кировский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Колпинский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Красногвардейский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Красносельский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Кронштадтский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Курортный район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Ломоносовский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Московский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Невский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Петроградский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Петродворцовый район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Приморский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Пушкинский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Фрунзенский район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "Центральный район", "admin_level": 5, "parent": "Санкт-Петербург"},
    {"name": "мун. округ Коломна", "admin_level": 6, "parent": "Адмиралтейский район"},
    {"name": "мун. округ Сенной округ", "admin_level": 6, "parent": "Адмиралтейский район"},
    {"name": "мун. округ Адмиралтейский округ", "admin_level": 6, "parent": "Адмиралтейский район"},
    {"name": "мун. округ Семёновский", "admin_level": 6, "parent": "Адмиралтейский район"},
    {"name": "мун. округ Измайловское", "admin_level": 6, "parent": "Адмиралтейский район"},
    {"name": "мун. округ Екатерингофский", "admin_level": 6, "parent": "Адмиралтейский район"},
    {"name": "мун. округ № 7", "admin_level": 6, "parent": "Василеостровский район"},
    {"name": "мун. округ Васильевский", "admin_level": 6, "parent": "Василеостровский район"},
    {"name": "мун. округ Гавань", "admin_level": 6, "parent": "Василеостровский район"},
    {"name": "мун. округ Морской", "admin_level": 6, "parent": "Василеостровский район"},
    {"name": "nan округ Морской", "admin_level": 6, "parent": "nan район"},
    {"name": "мун. округ Остров Декабристов", "admin_level": 6, "parent": "Василеостровский район"},
    {"name": "мун. округ Сампсониевское", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "мун. округ Светлановское", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "мун. округ Сосновское", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "мун. округ № 15", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "мун. округ Сергиевское", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "nan Парнас", "admin_level": 6, "parent": "nan район"},
    {"name": "мун. округ Шувалово-Озерки", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "посёлок Левашово", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "посёлок Парголово", "admin_level": 6, "parent": "Выборгский район"},
    {"name": "мун. округ Гражданка", "admin_level": 6, "parent": "Калининский район"},
    {"name": "мун. округ Академическое", "admin_level": 6, "parent": "Калининский район"},
    {"name": "мун. округ Финляндский округ", "admin_level": 6, "parent": "Калининский район"},
    {"name": "мун. округ № 21", "admin_level": 6, "parent": "Калининский район"},
    {"name": "мун. округ Пискарёвка", "admin_level": 6, "parent": "Калининский район"},
    {"name": "мун. округ Северный", "admin_level": 6, "parent": "Калининский район"},
    {"name": "мун. округ Прометей", "admin_level": 6, "parent": "Калининский район"},
    {"name": "мун. округ Княжево", "admin_level": 6, "parent": "Кировский район"},
    {"name": "мун. округ Ульянка", "admin_level": 6, "parent": "Кировский район"},
    {"name": "мун. округ Дачное", "admin_level": 6, "parent": "Кировский район"},
    {"name": "мун. округ Автово", "admin_level": 6, "parent": "Кировский район"},
    {"name": "мун. округ Нарвский округ", "admin_level": 6, "parent": "Кировский район"},
    {"name": "мун. округ Красненькая речка", "admin_level": 6, "parent": "Кировский район"},
    {"name": "мун. округ Морские ворота", "admin_level": 6, "parent": "Кировский район"},
    {"name": "город Колпино", "admin_level": 6, "parent": "Колпинский район"},
    {"name": "посёлок Понтонный", "admin_level": 6, "parent": "Колпинский район"},
    {"name": "посёлок Усть-Ижора", "admin_level": 6, "parent": "Колпинский район"},
    {"name": "посёлок Петро-Славянка", "admin_level": 6, "parent": "Колпинский район"},
    {"name": "посёлок Сапёрный", "admin_level": 6, "parent": "Колпинский район"},
    {"name": "посёлок Металлострой", "admin_level": 6, "parent": "Колпинский район"},
    {"name": "мун. округ Полюстрово", "admin_level": 6, "parent": "Красногвардейский район"},
    {"name": "мун. округ Большая Охта", "admin_level": 6, "parent": "Красногвардейский район"},
    {"name": "мун. округ Малая Охта", "admin_level": 6, "parent": "Красногвардейский район"},
    {"name": "мун. округ Пороховые", "admin_level": 6, "parent": "Красногвардейский район"},
    {"name": "мун. округ Ржевка", "admin_level": 6, "parent": "Красногвардейский район"},
    {"name": "мун. округ Юго-Запад", "admin_level": 6, "parent": "Красносельский район"},
    {"name": "мун. округ Южно-Приморский", "admin_level": 6, "parent": "Красносельский район"},
    {"name": "мун. округ Сосновая Поляна", "admin_level": 6, "parent": "Красносельский район"},
    {"name": "мун. округ Урицк", "admin_level": 6, "parent": "Красносельский район"},
    {"name": "мун. округ Константиновское", "admin_level": 6, "parent": "Красносельский район"},
    {"name": "мун. округ Горелово", "admin_level": 6, "parent": "Красносельский район"},
    {"name": "город Красное Село", "admin_level": 6, "parent": "Красносельский район"},
    {"name": "город Кронштадт", "admin_level": 6, "parent": "Кронштадтский район"},
    {"name": "город Зеленогорск", "admin_level": 6, "parent": "Курортный район"},
    {"name": "город Сестрорецк", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Белоостров", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Комарово", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Молодёжное", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Песочный", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Репино", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Серово", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Смолячково", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Солнечное", "admin_level": 6, "parent": "Курортный район"},
    {"name": "посёлок Ушково", "admin_level": 6, "parent": "Курортный район"},
    {"name": "мун. округ Московская застава", "admin_level": 6, "parent": "Московский район"},
    {"name": "мун. округ Гагаринское", "admin_level": 6, "parent": "Московский район"},
    {"name": "мун. округ Новоизмайловское", "admin_level": 6, "parent": "Московский район"},
    {"name": "мун. округ Пулковский меридиан", "admin_level": 6, "parent": "Московский район"},
    {"name": "мун. округ Звёздное", "admin_level": 6, "parent": "Московский район"},
    {"name": "мун. округ Невская застава", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Ивановский", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Обуховский", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Рыбацкое", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Народный", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ № 54", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Невский округ", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Оккервиль", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Правобережный", "admin_level": 6, "parent": "Невский район"},
    {"name": "мун. округ Введенский", "admin_level": 6, "parent": "Петроградский район"},
    {"name": "мун. округ Кронверкское", "admin_level": 6, "parent": "Петроградский район"},
    {"name": "мун. округ Посадский", "admin_level": 6, "parent": "Петроградский район"},
    {"name": "мун. округ округ Петровский", "admin_level": 6, "parent": "Петроградский район"},
    {"name": "мун. округ Чкаловское", "admin_level": 6, "parent": "Петроградский район"},
    {"name": "посёлок Стрельна", "admin_level": 6, "parent": "Петродворцовый район"},
    {"name": "город Ломоносов", "admin_level": 6, "parent": "Петродворцовый район"},
    {"name": "город Петергоф", "admin_level": 6, "parent": "Петродворцовый район"},
    {"name": "мун. округ Лахта-Ольгино", "admin_level": 6, "parent": "Приморский район"},
    {"name": "мун. округ № 65", "admin_level": 6, "parent": "Приморский район"},
    {"name": "мун. округ Ланское", "admin_level": 6, "parent": "Приморский район"},
    {"name": "nan Чёрная речка", "admin_level": 6, "parent": "nan район"},
    {"name": "мун. округ Комендантский аэродром", "admin_level": 6, "parent": "Приморский район"},
    {"name": "мун. округ Озеро Долгое", "admin_level": 6, "parent": "Приморский район"},
    {"name": "мун. округ Аптекарский остров", "parent": "Приморский район"},
    {"name": "мун. округ Юнтолово", "admin_level": 6, "parent": "Приморский район"},
    {"name": "мун. округ Коломяги", "admin_level": 6, "parent": "Приморский район"},
    {"name": "посёлок Лисий Нос", "admin_level": 6, "parent": "Приморский район"},
    {"name": "город Павловск", "admin_level": 6, "parent": "Пушкинский район"},
    {"name": "город Пушкин", "admin_level": 6, "parent": "Пушкинский район"},
    {"name": "посёлок Шушары", "admin_level": 6, "parent": "Пушкинский район"},
    {"name": "посёлок Александровская", "admin_level": 6, "parent": "Пушкинский район"},
    {"name": "посёлок Тярлево", "admin_level": 6, "parent": "Пушкинский район"},
    {"name": "мун. округ Волковское", "admin_level": 6, "parent": "Фрунзенский район"},
    {"name": "мун. округ № 72", "admin_level": 6, "parent": "Фрунзенский район"},
]

CITY_SERVICES_NAMES = [
    "Детский сад",
    "Школа",
    "Сад",
    "Дом детского творчества",
    "Детско-юношеская спортивная школа",
    "Детский лагерь",
    "Среднее специальное учебное заведение",
    "Высшее учебное заведение",
    "Поликлиника",
    "Детская поликлиника",
    "Стоматологическая клиника",
    "Фельдшерско-акушерский пункт",
    "Женская консультация",
    "Реабилитационный центр",
    "Аптека",
    "Метро",
    "Больница",
    "Роддом",
    "Детская больница",
    "Хоспис",
    "Станция скорой медицинской помощи",
    "Травматологические пункты",
    "Морг",
    "Диспансер",
    "Комплексный центр социального обслуживания населения",
    "Дом престарелых",
    "Центр занятости населения",
    "Детские дома-интернаты",
    "Многофункциональные центры предоставления государственных и муниципальных услуг",
    "Библиотека",
    "Дворец культуры",
    "Музей",
    "Театр",
    "Ботанический сад",
    "Концертный зал",
    "Цирк",
    "Зоопарк",
    "Кинотеатр",
    "Торговый центр",
    "Аквапарк",
    "Стадион",
    "Ледовая арена",
    "Кафе",
    "Ресторан",
    "Бар",
    "Столовая",
    "Булочная",
    "Парк",
    "Лес",
    "Заказник",
    "Заповедник",
    "Газон",
    "Поле",
    "Пляж",
    "Река",
    "Озеро",
    "Болото",
    "Ручей",
    "Залив",
    "Спортивная площадка",
    "Бассейн",
    "Спортивный зал",
    "Каток",
    "Футбольное поле",
    "Веревочный парк",
    "Экологическая тропа",
    "Скалодром",
    "Детская площадка",
    "Парк атракционов",
    "Игровое пространство",
    "Скейт-парк",
    "Полицейский участок",
    "Пожарная станция",
    "Железнодорожная станция",
    "Железнодорожный вокзал",
    "Аэропорт",
    "Аэродром",
    "Автозаправка",
    "Парковка",
    "Автовокзал",
    "Остановка наземного общественного транспорта",
    "Выход метро",
    "Супермаркет",
    "Продукты",
    "Рынок",
    "Хозяйственные товары",
    "Одежда и обувь",
    "Бытовая техника",
    "Книжный магазин",
    "Детские товары",
    "Спортивный магазин",
    "Почтовое отделение",
    "Пункт выдачи",
    "Отделение банка",
    "Банкомат",
    "Адвокат",
    "Нотариальная контора",
    "Парикмахер",
    "Салон красоты",
    "Общественная баня",
    "Ветеринарная клиника",
    "Зоомагазин",
    "Площадка для выгула собак",
    "Гостиница",
    "Хостел",
    "Фонтан",
    "База отдыха",
    "Памятник",
    "Церковь",
    "Кладбище",
    "Котельная",
    "Тепло-электроцентраль",
    "Атомная электростанция",
    "Гидро-электростанция",
    "Тепловая электростанция",
    "Солнечная батарея",
    "Ветрогенератор",
    "Электрическая подстанция",
    "Водозаборные сооружения",
    "Насосные станции водоснабжения",
    "Сооружения для очистки воды",
    "Водонапорные башни и резервуары",
    "Водоочистное сооружение",
    "Выпуски сточных вод в водоем",
    "Насосная станция водоотведения",
    "Воинская часть",
    "Промышленная зона",
    "Магазин",
    "Остановка",
    "ДК",
    "Детская площадка",
    "Сад",
    "Баня",
    "Университет",
    "Институт",
]

NUM_CITY_OBJ = {
    "школа": [
        "школа",
        "школы",
        "школе",
        "школу",
        "школой",
        "школе",
        "школы",
        "школ",
        "школам",
        "школы",
        "школами",
        "школах",
        "школ",
    ],
    "лицей": [
        "лицей",
        "лицея",
        "лицею",
        "лицей",
        "лицеем",
        "лицее",
        "лицеи",
        "лицеев",
        "лицеям",
        "лицеи",
        "лицеями",
        "лицеях",
        "лицеев",
    ],
    "гимназия": [
        "гимназия",
        "гимназии",
        "гимназии",
        "гимназию",
        "гимназией",
        "гимназии",
        "гимназии",
        "гимназий",
        "гимназиям",
        "гимназии",
        "гимназиями",
        "гимназиях",
        "гимназий",
    ],
    "поликлиника": [
        "поликлиника",
        "поликлиники",
        "поликлинике",
        "поликлинику",
        "поликлиникой",
        "поликлинике",
        "поликлиники",
        "поликлиник",
        "поликлиникам",
        "поликлиники",
        "поликлиниками",
        "поликлиниках",
        "поликлиник",
    ],
    "больница": [
        "больница",
        "больницы",
        "больнице",
        "больницу",
        "больницей",
        "больнице",
        "больницы",
        "больниц",
        "больницам",
        "больницы",
        "больницами",
        "больницах",
        "больниц",
    ],
    "городская больница": [
        "городская больница",
        "городской больницы",
        "городской больнице",
        "городскую больницу",
        "городской больницей",
        "городской больнице",
        "городские больницы",
        "городских больниц",
        "городским больницам",
        "городские больницы",
        "городскими больницами",
        "городских больницах",
        "городских больниц",
    ],
    "детский сад": [
        "детский сад",
        "детского сада",
        "детскому саду",
        "детский сад",
        "детским садом",
        "детском саде",
        "детские сады",
        "детских садов",
        "детским садам",
        "детские сады",
        "детскими садами",
        "детских садах",
        "детских садов",
    ],
    "стоматологическая поликлиника": [
        "стоматологическая поликлиника",
        "стоматологической поликлиники",
        "стоматологической поликлинике",
        "стоматологическую поликлинику",
        "стоматологической поликлиникой",
        "стоматологической поликлинике",
        "стоматологические поликлиники",
        "стоматологических поликлиник",
        "стоматологическим поликлиникам",
        "стоматологические поликлиники",
        "стоматологическими поликлиниками",
        "стоматологических поликлиниках",
        "стоматологических поликлиник",
    ],
    "детская поликлиника": [
        "детская поликлиника",
        "детской поликлиники",
        "детской поликлинике",
        "детскую поликлинику",
        "детской поликлиникой",
        "детской поликлинике",
        "детские поликлиники",
        "детских поликлиник",
        "детским поликлиникам",
        "детские поликлиники",
        "детскими поликлиниками",
        "детских поликлиниках",
        "детских поликлиник",
    ],
    "женская консультация": [
        "женская консультация",
        "женской консультации",
        "женской консультации",
        "женскую консультацию",
        "женской консультацией",
        "женской консультации",
        "женские консультации",
        "женских консультаций",
        "женским консультациям",
        "женские консультации",
        "женскими консультациями",
        "женских консультациях",
        "женских консультаций",
    ],
    "городская поликлиника": [
        "городская поликлиника",
        "городской поликлиники",
        "городской поликлинике",
        "городскую поликлинику",
        "городской поликлиникой",
        "городской поликлинике",
        "городские поликлиники",
        "городских поликлиник",
        "городским поликлиникам",
        "городские поликлиники",
        "городскими поликлиниками",
        "городских поликлиниках",
        "городских поликлиник",
    ],
    "детская городская поликлиника": [
        "детская городская поликлиника",
        "детской городской поликлиники",
        "детской городской поликлинике",
        "детскую городскую поликлинику",
        "детской городской поликлиникой",
        "детской городской поликлинике",
        "детские городские поликлиники",
        "детских городских поликлиник",
        "детским городским поликлиникам",
        "детские городские поликлиники",
        "детскими городскими поликлиниками",
        "детских городских поликлиниках",
        "детских городских поликлиник",
    ],
}

EXCEPTIONS_CITY_COUNTRY = [
    "Санкт-Петербург",
    "СПБ",
    "спб",
    "санкт-петербург",
    "питер",
    "Питербург",
    "Питер",
    "питербург",
    "Москва",
    "москва",
    "мск",
    "МСК",
    "Петербург",
    "петербург ",
    "ФОЛКЛЕНДСКИЕ О-ВА",
    "МИКРОНЕЗИЯ",
    "ФАРЕРСКИЕ О-ВА",
    "ФРАНЦИЯ",
    "ГАБОН",
    "ВЕЛИКОБРИТАНИЯ",
    "ГРЕНАДА",
    "ГРУЗИЯ",
    "ГВИАНА",
    "ГАНА",
    "ГИБРАЛТАР",
    "ГРЕНЛАНДИЯ",
    "ГАМБИЯ",
    "ГВИНЕЯ",
    "ГВАДЕЛУПА",
    "ЭКВАТОР. ГВИНЕЯ",
    "ГРЕЦИЯ",
    "Ю.ДЖ.И САНДВ.О-ВА",
    "ГВАТЕМАЛА",
    "ГУАМ",
    "ГВИНЕЯ-БИСАУ",
    "ГАЙАНА",
    "СЯНГАН (ГОНКОНГ)",
    "ХЕРД И МАКДОНАЛЬД",
    "ГОНДУРАС",
    "ХОРВАТИЯ",
    "ГАИТИ",
    "ВЕНГРИЯ",
    "ИНДОНЕЗИЯ",
    "ИРЛАНДИЯ",
    "ИЗРАИЛЬ",
    "О-В МЭН",
    "ИНДИЯ",
    "БРИТ.ТЕР.В ИНД.ОК",
    "ИРАК",
    "ИРАН",
    "ИСЛАНДИЯ",
    "ИТАЛИЯ",
    "ЯМАЙКА",
    "ИОРДАНИЯ",
    "ЯПОНИЯ",
    "ДЖОНСТОН АТОЛЛ",
    "КЕНИЯ",
    "КЫРГЫЗСТАН",
    "КАМБОДЖА",
    "КИРИБАТИ",
    "КОМОРСКИЕ О-ВА",
    "СЕНТ-КИТС И НЕВИС",
    "КОРЕЯ (КНДР)",
    "КОРЕЯ РЕСП.",
    "КУВЕЙТ",
    "КАЙМАН",
    "КАЗАХСТАН",
    "ЛАОС",
    "ЛИВАН",
    "СЕНТ-ЛЮСИЯ",
    "ЛИХТЕНШТЕЙН",
    "ШРИ-ЛАНКА",
    "ЛИБЕРИЯ",
    "ЛЕСОТО",
    "ЛИТВА",
    "ЛЮКСЕМБУРГ",
    "ЛАТВИЯ",
    "ЛИВИЯ",
    "МАРОККО",
    "МОНАКО",
    "МОЛДОВА",
    "МАДАГАСКАР",
    "МАРШАЛЛОВЫ О-ВА",
    "О-ВА МИДУЭЙ",
    "МАКЕДОНИЯ",
    "МАЛИ",
    "МОНГОЛИЯ",
    "АОМЫНЬ(МАКАО)",
    "МАРИАНСКИЕ О-ВА",
    "МАРТИНИКА",
    "МАВРИТАНИЯ",
    "МОНТСЕРРАТ",
    "МАЛЬТА",
    "МАВРИКИЙ",
    "МАЛЬДИВЫ",
    "МАЛАВИ",
    "МЕКСИКА",
    "МАЛАЙЗИЯ",
    "МОЗАМБИК",
    "НАМИБИЯ",
    "НОВ.КАЛЕДОНИЯ",
    "НИГЕР",
    "НОРФОЛК",
    "НИГЕРИЯ",
    "НИКАРАГУА",
    "НИДЕРЛАНДЫ",
    "НОРВЕГИЯ",
    "НЕПАЛ",
    "НАУРУ",
    "НИУЭ",
    "НОВАЯ ЗЕЛАНДИЯ",
    "ОМАН",
    "ПАНАМА",
    "ПЕРУ",
    "ФР. ПОЛИНЕЗИЯ",
    "ПАПУА-НОВ.ГВИНЕЯ",
    "ФИЛИППИНЫ",
    "ПАКИСТАН",
    "ПОЛЬША",
    "С-ПЬЕР И МИКЕЛОН",
    "ПИТКЭРН",
    "ПУЭРТО-РИКО",
    "ПОРТУГАЛИЯ",
    "ПАЛАУ",
    "ПАРАГВАЙ",
    "КАТАР",
    "РЕЮНЬОН",
    "РУМЫНИЯ",
    "РОССИЯ",
    "РУАНДА",
    "САУДОВСКАЯ АРАВИЯ",
    "СОЛОМОНОВЫ О-ВА",
    "СЕЙШЕЛЬСКИЕ О-ВА",
    "СУДАН",
    "ШВЕЦИЯ",
    "СИНГАПУР",
    "О-В СВЯТОЙ ЕЛЕНЫ",
    "СЛОВЕНИЯ",
    "ШПИЦБЕРГЕН О-ВА",
    "СЛОВАКИЯ",
    "СЬЕРРА-ЛЕОНЕ",
    "САН-МАРИНО",
    "СЕНЕГАЛ",
    "СОМАЛИ",
    "СУРИНАМ",
    "САН-ТОМЕ И ПРИНС.",
    "САЛЬВАДОР",
    "СИРИЯ",
    "СВАЗИЛЕНД",
    "ТЕРКС И КАЙКОС",
    "ЧАД",
    "ФР.ЮЖНЫЕ ТЕРИТОР.",
    "ТОГО",
    "ТАИЛАНД",
    "ТАДЖИКИСТАН",
    "ТОКЕЛАУ (ЮНИОН)",
    "ТУРКМЕНИСТАН",
    "ТУНИС",
    "ТОНГА",
    "ВОСТОЧНЫЙ ТИМОР",
    "ТУРЦИЯ",
    "ТРИНИДАД И ТОБАГО",
    "ТУВАЛУ",
    "ТАЙВАНЬ",
    "ТАНЗАНИЯ",
    "УКРАИНА",
    "УГАНДА",
    "МАЛЫЕ ТИХООК.О-ВА",
    "США",
    "УРУГВАЙ",
    "УЗБЕКИСТАН",
    "ВАТИКАН",
    "С.ВИНСЕНТ.ГРЕНАД.",
    "ВЕНЕСУЭЛА",
    "ВИРГИН.О-ВА(БРИТ)",
    "ВИРГИН.О-ВА (США)",
    "ВЬЕТНАМ",
    "ВАНУАТУ",
    "УОЛЛИС И ФУТУНА",
    "О-В УЭЙК",
    "ЗАПАДНОЕ САМОА",
    "ЙЕМЕН",
    "ЮГОСЛАВИЯ",
    "ЮЖНО-АФР.РЕСПУБЛ.",
    "ЗАМБИЯ",
    "ЗАИР",
    "ЗИМБАБВЕ",
    "МЬЯНМА",
    "МАЙОТТА",
    "КОНГО",
    "ЭЛАНДСКИЕ О-ВА",
    "ПАЛЕСТИНА ОКУП.",
    "ТИМОР-ЛЕСТЕ",
    "ЧЕРНОГОРИЯ",
    "МЕЖДУНАРОДНЫЕ ОРГ",
    "ДЖЕРСИ",
    "Кюрасао",
    "ГЕРНСИ",
    "ДРУГАЯ СТРАНА",
    "АБХАЗИЯ",
    "СЕРБИЯ",
    "СТРАНА НЕ ОПР.",
    "АНДОРРА",
    "ОБЪЕД.РАБ.ЭМИРАТ",
    "АФГАНИСТАН",
    "АНТИГУА.БАРБ",
    "АНГИЛЬЯ",
    "АЛБАНИЯ",
    "АРМЕНИЯ",
    "АНТИЛЬСКИЕ О-ВА",
    "АНГОЛА",
    "АНТАРКТИКА",
    "АРГЕНТИНА",
    "ВОСТ.САМОА (США)",
    "АВСТРИЯ",
    "АВСТРАЛИЯ",
    "АРУБА",
    "АЗЕРБАЙДЖАН",
    "БОСНИЯ И ГЕРЦОГ.",
    "БАРБАДОС",
    "БАНГЛАДЕШ",
    "БЕЛЬГИЯ",
    "БУРКИНА-ФАСО",
    "БОЛГАРИЯ",
    "БАХРЕЙН",
    "БУРУНДИ",
    "БЕНИН",
    "БЕРМУДСКИЕ О-ВА",
    "БРУНЕЙ",
    "БОЛИВИЯ",
    "БРАЗИЛИЯ",
    "БАГАМСКИЕ О-ВА",
    "БУТАН",
    "БУВЕ",
    "БОТСВАНА",
    "БЕЛАРУСЬ",
    "БЕЛИЗ",
    "КАНАДА",
    "КОКОСОВЫЕ О-ВА",
    "ЦЕНТР.АФР.РЕСПУБ.",
    "КОНГО",
    "ШВЕЙЦАРИЯ",
    "КОТ-Д'ИВУАР",
    "О-ВА КУКА",
    "ЧИЛИ",
    "КАМЕРУН",
    "КИТАЙ",
    "КОЛУМБИЯ",
    "КОСТА-РИКА",
    "КУБА",
    "КАБО-ВЕРДЕ",
    "О-В РОЖДЕСТВА",
    "КИПР",
    "ЧЕХИЯ",
    "ГЕРМАНИЯ",
    "ДЖИБУТИ",
    "ДАНИЯ",
    "ДОМИНИКА",
    "ДОМИНИКАНСК.РЕСП.",
    "АЛЖИР",
    "ЭКВАДОР",
    "ЭСТОНИЯ",
    "ЕГИПЕТ",
    "ЗАПАДНАЯ САХАРА",
    "ЭРИТРЕЯ",
    "ИСПАНИЯ",
    "ЭФИОПИЯ",
    "ФИНЛЯНДИЯ",
    "ФИДЖИ",
]

AREA_STOPWORDS = ["сельское поселение", "городское поселение", "район", "округ"]

GROUP_STOPWORDS = [
    "сельское поселение",
    "городское поселение",
    "округ",
    "город",
    "муниципальное образование",
    "Муниципальный",
    "Администрация",
    " МО",
    "МО ",
]

REGEX_PATTERN = r"[\"!?\u2665\u2022()|,.-:]"

REPLACEMENT_STRING = " "



"""
This module contains classes for retrieving and working with various types of data.

@class:GeoDataGetter:
This class is used to retrieve geospatial data from OpenStreetMap (OSM) based on given OSM ID and tags.

@class:VKParser:
A class for parsing and working with VK comments and posts. Combines posts and comments into one dataframe.

@class:Streets:
A class for working with street data.

"""
import pandas as pd
from tqdm import tqdm
import requests
import datetime
import time
import random

class VKParser:
    API_VERISON = "5.131"
    COUNT_ITEMS = 100
    # SLEEP_TIME = 0.5
    TIMEOUT_LIMIT = 15

    @staticmethod
    def get_group_name(domain, accsess_token):
        params = {"group_id": domain, "access_token": accsess_token, "v": VKParser.API_VERISON}
        response = requests.get("https://api.vk.com/method/groups.getById", params=params)  # передвинуть повыше
        data = response.json()
        if "response" in data and data["response"]:
            group_name = data["response"][0]["name"]
            return pd.DataFrame({"group_name": [group_name]})
        else:
            print("Error while fetching group name:", data)
            return pd.DataFrame({"group_name": [None]})

    @staticmethod
    def get_owner_id_by_domain(domain, access_token):
        """
        Get the owner ID of a VK group by its domain.

        Args:
            domain (str): The domain of the VK group.
            access_token (str): The access token for the VK API.

        Returns:
            int: The owner ID of the VK group, or None if the request was not successful.
        """
        url = "https://api.vk.com/method/wall.get"
        params = {
            "domain": domain,
            "access_token": access_token,
            "v": VKParser.API_VERISON,
        }
        response = requests.get(url, params=params)
        if response.ok:
            owner_id = response.json()["response"]["items"][0]["owner_id"]
        else:
            owner_id = None
        return owner_id

    @staticmethod
    def get_group_post_ids(domain, access_token, post_num_limit, step) -> list:
        """
        A static method to retrieve a list of post IDs for a given group, based on the owner ID,
        access token, post number limit, and step size. Returns a list of post IDs.
        """
        offset = 0
        post_ids = []

        while offset < post_num_limit:
            print(offset, " | ", post_num_limit, end="\r")
            res = requests.get(
                "https://api.vk.com/method/wall.get",
                params={
                    "access_token": access_token,
                    "v": VKParser.API_VERISON,
                    "domain": domain,
                    "count": step,
                    "offset": offset,
                },
                timeout=10,
            ).json()["response"]
            # print(res.json().keys())
            time.sleep(random.random())

            post_ids_new = [k["id"] for k in res["items"]]
            post_ids += post_ids_new
            offset += step

        return post_ids

    @staticmethod
    def get_subcomments(params):
        """
        Retrieves subcomments from the VK API.

        Args:
            owner_id (int): The ID of the owner of the comments.
            post_id (int): The ID of the post.
            access_token (str): The access token for authentication.
            params (dict): Additional parameters for the API request.

        Returns:
            list: A list of subcomments retrieved from the API.
        """
        subcomments = []

        response = requests.get("https://api.vk.com/method/wall.getComments", params=params)
        # print(response.json().keys())
        time.sleep(random.random())
        data = response.json()

        if "response" in data:
            for item in data["response"]["items"]:
                item["date"] = datetime.datetime.utcfromtimestamp(item["date"]).strftime("%Y-%m-%d %H:%M:%S")
                if "likes" in item:
                    item["likes.count"] = item["likes"]["count"]
                subcomments.append(item)

        return subcomments

    def get_comments(self, owner_id, post_id, access_token):
        """
        Get comments for a post on VK using the specified owner ID, post ID, and access token.

        Parameters:
            owner_id (int): The ID of the post owner.
            post_id (int): The ID of the post.
            access_token (str): The access token for authentication.

        Returns:
            list: A list of dictionaries containing comment information.
        """
        params = {
            "owner_id": owner_id,
            "post_id": post_id,
            "access_token": access_token,
            "v": VKParser.API_VERISON,
            "extended": 1,
            "count": 100,
            "need_likes": 1,
        }

        comments = []

        response = requests.get("https://api.vk.com/method/wall.getComments", params=params)
        # print(response.json().keys())
        time.sleep(random.random())
        data = response.json()

        if "response" in data:
            for item in data["response"]["items"]:
                if item["text"] == "":
                    continue
                item["date"] = datetime.datetime.utcfromtimestamp(item["date"]).strftime("%Y-%m-%d %H:%M:%S")
                if "likes" in item:
                    item["likes.count"] = item["likes"]["count"]
                comments.append(item)
                if item["thread"]["count"] > 0:
                    params["comment_id"] = item["id"]
                    subcomments = VKParser.get_subcomments(params)
                    comments.extend(subcomments)
        return comments

    @staticmethod
    def comments_to_dataframe(comments):
        """
        Convert comments to a DataFrame.

        Args:
            comments: List of comments to be converted.

        Returns:
            DataFrame: A DataFrame containing specific columns from the input comments.
        """
        df = pd.DataFrame(comments)
        df = df[["id", "from_id", "date", "text", "post_id", "parents_stack", "likes.count"]]
        return df

    @staticmethod
    def run_posts(domain, access_token, cutoff_date, number_of_messages=float("inf"), step=50):
        """
        A function to retrieve posts from a social media API based on specified parameters.

        Parameters:
            owner_id (int): The ID of the owner whose posts are being retrieved.
            access_token (str): The authentication token for accessing the API.
            step (int): The number of posts to retrieve in each API call.
            cutoff_date (str): The date to stop retrieving posts (format: '%Y-%m-%d').
            number_of_messages (float): The maximum number of messages to retrieve (default is infinity).

        Returns:
            pandas.DataFrame: A DataFrame containing the retrieved posts.
        """
    
        offset = 0
        all_posts = []
        if step > number_of_messages:
            step = number_of_messages
        while offset < number_of_messages:
            print(offset, " | ", number_of_messages, end="\r")

            response = requests.get(
                "https://api.vk.com/method/wall.get",
                params={
                    "access_token": access_token,
                    "v": VKParser.API_VERISON,
                    "domain": domain,
                    "count": step,
                    "offset": offset,
                }, timeout=600
            )
            if response.ok:
                # print(response.json().keys())
                data = response.json()["response"]["items"]
                offset += step
                current_posts = pd.json_normalize(data)
                current_posts = current_posts[["date", "id", "text", "views.count", "likes.count", "reposts.count"]]
                current_posts["date"] = [
                    datetime.datetime.fromtimestamp(current_posts["date"][i]) for i in range(len(current_posts["date"]))
                ]
                current_posts["type"] = "post"
                all_posts.append(current_posts)
                print(current_posts.date.min())
                if any(current_posts["date"] < datetime.datetime.strptime(cutoff_date, "%Y-%m-%d")):
                    print("posts downloaded")
                    break
            else:
                continue
            time.sleep(random.random())
        df_posts = pd.concat(all_posts).reset_index(drop=True)
        df_posts = df_posts[df_posts.text.map(lambda x: len(x)) > 0]
        df_posts["text"] = df_posts["text"].str.replace(r"\n", "", regex=True)
        df_posts["link"] = df_posts["text"].str.extract(r"(https://\S+)")
        return df_posts
    
    @staticmethod
    def run_comments(domain, post_ids, access_token):
        owner_id = VKParser.get_owner_id_by_domain(domain, access_token)
        all_comments = []
        for post_id in tqdm(post_ids):
            comments = VKParser().get_comments(owner_id, post_id, access_token)
            all_comments.extend(comments)
        if len(all_comments) > 0:
            df = VKParser.comments_to_dataframe(all_comments)
            df["type"] = "comment"
            df = df.reset_index(drop=True)
            print("comments downloaded")
            return df
        else:
            print("no comments")
            return None

    @staticmethod
    def run_parser(domain, access_token, cutoff_date, number_of_messages=float("inf"), step=100):
        """
        Runs the parser with the given parameters and returns a combined DataFrame of posts and comments.

        :param owner_id: The owner ID for the parser.
        :param access_token: The user token for authentication.
        :param step: The step size for fetching data.
        :param cutoff_date: The cutoff date for fetching data.
        :param number_of_messages: The maximum number of messages to fetch. Defaults to positive infinity.
        :return: A combined DataFrame of posts and comments.
        """
        owner_id = VKParser.get_owner_id_by_domain(domain, access_token)
        df_posts = VKParser.run_posts(domain=owner_id, access_token=access_token, step=step, cutoff_date=cutoff_date, number_of_messages=number_of_messages)
        post_ids = df_posts["id"].tolist()

        df_comments = VKParser.run_comments(domain=owner_id, post_ids=post_ids, access_token=access_token)
        if df_comments is not None:
            df_comments.loc[df_comments["parents_stack"].apply(lambda x: len(x) > 0), "type"] = "reply"
            for i in range(len(df_comments)):
                tmp = df_comments["parents_stack"].iloc[i]
                if tmp is not None:
                    if len(tmp) > 0:
                        df_comments["parents_stack"].iloc[i] = tmp[0]
                    else:
                        df_comments["parents_stack"].iloc[i] = None

            df_combined = df_comments.join(df_posts, on="post_id", rsuffix="_post")
            df_combined = pd.concat([df_posts, df_comments], ignore_index=True)
        else:
            df_combined = df_posts
        df_group_name = VKParser.get_group_name(domain, access_token)
        df_combined["group_name"] = df_group_name["group_name"][0]

        return df_combined



import pandas as pd
from typing import List, Optional
from loguru import logger

class WordFormFinder:
    def __init__(self, osm_city_name: str):
        self.osm_city_name = osm_city_name

    def find_word_form(self, df: pd.DataFrame, strts_df: pd.DataFrame) -> pd.DataFrame:
        """
        Match free form street names extracted from social media texts to the standardized forms used in the OSM database.

        This function normalizes and finds the full street names by matching extracted street names and 
        their toponyms with entries in the OSM database.

        Args:
            df (pd.DataFrame): DataFrame containing extracted street names and toponyms.
            strts_df (pd.DataFrame): DataFrame containing standardized street names and toponyms from OSM.

        Returns:
            pd.DataFrame: DataFrame with matched full street names and their variations.
        """
        results = []

        for idx, row in df.iterrows():
            result = self._process_row(row, strts_df)
            results.append(result)

        # Create a new DataFrame from results
        result_df = self._results_to_dataframe(df, results)

        return result_df

    def _process_row(self, row: pd.Series, strts_df: pd.DataFrame) -> dict:
        """
        Process a single row to find matching full street names.

        Args:
            row (pd.Series): The row to process.
            strts_df (pd.DataFrame): DataFrame containing standardized street names and toponyms from OSM.

        Returns:
            dict: A dictionary with the results for the row.
        """
        try:
            search_val = row.get("Street")
            search_toponym = row.get("Toponyms", "")
            val_num = row.get("Numbers", "")

            if not search_val or pd.isna(search_val):
                logger.warning(f"Error processing row with street '{row.get('Street')}' and toponym '{row.get('Toponyms')}")
                return {"full_street_name": None, "only_full_street_name": None}

            for col in strts_df.columns[2:]:
                matching_rows = self._find_matching_rows(strts_df, col, search_val, search_toponym)

                if not matching_rows.empty:
                    full_streets = [self._format_full_address(street, val_num) for street in matching_rows["street"].values]
                    return {
                        "full_street_name": ",".join(full_streets),
                        "only_full_street_name": ",".join(matching_rows["street"].values)
                    }

            # If no exact match found, check without toponym
            for col in strts_df.columns[2:]:
                if search_val in strts_df[col].values:
                    only_streets_full = strts_df.loc[strts_df[col] == search_val, "street"].values
                    full_streets = [self._format_full_address(street, val_num) for street in only_streets_full]
                    return {
                        "full_street_name": ";".join(full_streets),
                        "only_full_street_name": ";".join(only_streets_full)
                    }

            logger.warning(f"Error processing row with street '{row.get('Street')}' and toponym '{row.get('Toponyms')}'")
            return {"full_street_name": None, "only_full_street_name": None}

        except Exception as e:
            logger.warning(f"Error processing row with street '{row.get('Street')}' and toponym '{row.get('Toponyms')}': {e}")

            return {"full_street_name": None, "only_full_street_name": None}

    def _find_matching_rows(self, strts_df: pd.DataFrame, col: str, search_val: str, search_toponym: Optional[str]) -> pd.DataFrame:
        """
        Find rows in the OSM DataFrame that match the search value and toponym.

        Args:
            strts_df (pd.DataFrame): DataFrame containing standardized street names and toponyms from OSM.
            col (str): The column to search in.
            search_val (str): The street name to search for.
            search_toponym (Optional[str]): The toponym to match.

        Returns:
            pd.DataFrame: DataFrame with matching rows.
        """
        search_rows = strts_df.loc[strts_df[col] == search_val]

        if search_toponym:
            return search_rows[search_rows["toponim_name"] == search_toponym]
        else:
            return search_rows

    def _format_full_address(self, street: str, val_num: str) -> str:
        """
        Format the full address with the building number, city, and country.

        Args:
            street (str): The street name.
            val_num (str): Building number or additional value.

        Returns:
            str: Formatted full address.
        """
        return f"{street} {val_num} {self.osm_city_name} Россия"

    def _results_to_dataframe(self, df: pd.DataFrame, results: List[dict]) -> pd.DataFrame:
        """
        Convert results list to a DataFrame and merge it with the original DataFrame.

        Args:
            df (pd.DataFrame): Original DataFrame.
            results (List[dict]): List of result dictionaries.

        Returns:
            pd.DataFrame: DataFrame with additional columns for full street names.
        """
        results_df = pd.DataFrame.from_records(results)
        merged_df = df.reset_index(drop=True).join(results_df)

        # Explode lists into rows and merge them back into the DataFrame
        merged_df.dropna(subset=["full_street_name", "only_full_street_name"], inplace=True)

        merged_df["location_options"] = merged_df["full_street_name"].str.split(";")
        merged_df["only_full_street_name"] = merged_df["only_full_street_name"].str.split(";")

        exploded_locations = merged_df["location_options"].explode().rename("addr_to_geocode")
        exploded_streets = merged_df["only_full_street_name"].explode().rename("only_full_street_name")
        exploded_locations = exploded_locations.to_frame()
        exploded_streets = exploded_streets.to_frame()
        final_df = exploded_locations.join(exploded_streets)

        merged_df.drop(columns=["only_full_street_name"], inplace=True)
        final_df = merged_df.merge(final_df, left_index=True, right_index=True)

        return final_df
    



from yargy import (
    rule,
    or_, and_
)
from yargy.interpretation import fact
from yargy.predicates import (
    eq, lte, gte, gram, type, tag,
    length_eq,
    in_, in_caseless, dictionary,
    normalized, caseless,
    is_title
)
from yargy.pipelines import morph_pipeline
from yargy.tokenizer import QUOTES


Index = fact(
    'Index',
    ['value']
)
Region = fact(
    'Region',
    ['name', 'type']
)
Raion = fact(
    'Raion',
    ['name', 'type']
)
Settlement = fact(
    'Settlement',
    ['name', 'type']
)
Street = fact(
    'Street',
    ['name', 'type']
)
Building = fact(
    'Building',
    ['number', 'type']
)
Room = fact(
    'Room',
    ['number', 'type']
)
AddrPart = fact(
    'AddrPart',
    ['value']
)


def value(key):
    @property
    def field(self):
        return getattr(self, key)
    return field


class Index(Index):
    type = 'индекс'




class Region(Region):
    value = value('name')


class Raion(Raion):
    value = value('name')


class Settlement(Settlement):
    value = value('name')


class Street(Settlement):
    value = value('name')


class Building(Building):
    value = value('number')


class Room(Room):
    value = value('number')


class AddrPart(AddrPart):
    @property
    def obj(self):
        from natasha import obj

        part = self.value
        return obj.AddrPart(part.value, part.type)


DASH = eq('-')
DOT = eq('.')

ADJF = gram('ADJF')
NOUN = gram('NOUN')
INT = type('INT')
TITLE = is_title()

ANUM = rule(
    INT,
    DASH.optional(),
    in_caseless({
        'я', 'й', 'е',
        'ое', 'ая', 'ий', 'ой'
    })
)




#############
#
#  FED OKRUGA
#
############


FED_OKRUG_NAME = or_(
    rule(
        dictionary({
            'дальневосточный',
            'приволжский',
            'сибирский',
            'уральский',
            'центральный',
            'южный',
        })
    ),
    rule(
        caseless('северо'),
        DASH.optional(),
        dictionary({
            'западный',
            'кавказский'
        })
    )
).interpretation(
    Region.name
)

FED_OKRUG_WORDS = or_(
    rule(
        normalized('федеральный'),
        normalized('округ')
    ),
    rule(caseless('фо'))
).interpretation(
    Region.type.const('федеральный округ')
)

FED_OKRUG = rule(
    FED_OKRUG_WORDS,
    FED_OKRUG_NAME
).interpretation(
    Region
)


#########
#
#   RESPUBLIKA
#
############


RESPUBLIKA_WORDS = or_(
    rule(caseless('респ'), DOT.optional()),
    rule(normalized('республика'))
).interpretation(
    Region.type.const('республика')
)

RESPUBLIKA_ADJF = or_(
    rule(
        dictionary({
            'удмуртский',
            'чеченский',
            'чувашский',
        })
    ),
    rule(
        caseless('карачаево'),
        DASH.optional(),
        normalized('черкесский')
    ),
    rule(
        caseless('кабардино'),
        DASH.optional(),
        normalized('балкарский')
    )
).interpretation(
    Region.name
)

RESPUBLIKA_NAME = or_(
    rule(
        dictionary({
            'адыгея',
            'алтай',
            'башкортостан',
            'бурятия',
            'дагестан',
            'ингушетия',
            'калмыкия',
            'карелия',
            'коми',
            'крым',
            'мордовия',
            'татарстан',
            'тыва',
            'удмуртия',
            'хакасия',
            'саха',
            'якутия',
        })
    ),
    rule(caseless('марий'), caseless('эл')),
    rule(
        normalized('северный'), normalized('осетия'),
        rule('-', normalized('алания')).optional()
    )
).interpretation(
    Region.name
)

RESPUBLIKA_ABBR = in_caseless({
    'кбр',
    'кчр',
    'рт',  # Татарстан
}).interpretation(
    Region.name  # TODO type
)

RESPUBLIKA = or_(
    rule(RESPUBLIKA_ADJF, RESPUBLIKA_WORDS),
    rule(RESPUBLIKA_WORDS, RESPUBLIKA_NAME),
    rule(RESPUBLIKA_ABBR)
).interpretation(
    Region
)


##########
#
#   KRAI
#
########


KRAI_WORDS = normalized('край').interpretation(
    Region.type.const('край')
)

KRAI_NAME = dictionary({
    'алтайский',
    'забайкальский',
    'камчатский',
    'краснодарский',
    'красноярский',
    'пермский',
    'приморский',
    'ставропольский',
    'хабаровский',
}).interpretation(
    Region.name
)

KRAI = rule(
    KRAI_NAME, KRAI_WORDS
).interpretation(
    Region
)


############
#
#    OBLAST
#
############


OBLAST_WORDS = or_(
    rule(normalized('область')),
    rule(
        caseless('обл'),
        DOT.optional()
    )
).interpretation(
    Region.type.const('область')
)

OBLAST_NAME = dictionary({
    'амурский',
    'архангельский',
    'астраханский',
    'белгородский',
    'брянский',
    'владимирский',
    'волгоградский',
    'вологодский',
    'воронежский',
    'горьковский',
    'ивановский',
    'ивановский',
    'иркутский',
    'калининградский',
    'калужский',
    'камчатский',
    'кемеровский',
    'кировский',
    'костромской',
    'курганский',
    'курский',
    'ленинградский',
    'липецкий',
    'магаданский',
    'московский',
    'мурманский',
    'нижегородский',
    'новгородский',
    'новосибирский',
    'омский',
    'оренбургский',
    'орловский',
    'пензенский',
    'пермский',
    'псковский',
    'ростовский',
    'рязанский',
    'самарский',
    'саратовский',
    'сахалинский',
    'свердловский',
    'смоленский',
    'тамбовский',
    'тверской',
    'томский',
    'тульский',
    'тюменский',
    'ульяновский',
    'челябинский',
    'читинский',
    'ярославский',
}).interpretation(
    Region.name
)

OBLAST = rule(
    OBLAST_NAME,
    OBLAST_WORDS
).interpretation(
    Region
)


##########
#
#    AUTO OKRUG
#
#############


AUTO_OKRUG_NAME = or_(
    rule(
        dictionary({
            'чукотский',
            'эвенкийский',
            'корякский',
            'ненецкий',
            'таймырский',
            'агинский',
            'бурятский',
        })
    ),
    rule(caseless('коми'), '-', normalized('пермяцкий')),
    rule(caseless('долгано'), '-', normalized('ненецкий')),
    rule(caseless('ямало'), '-', normalized('ненецкий')),
).interpretation(
    Region.name
)

AUTO_OKRUG_WORDS = or_(
    rule(
        normalized('автономный'),
        normalized('округ')
    ),
    rule(caseless('ао'))
).interpretation(
    Region.type.const('автономный округ')
)

HANTI = rule(
    caseless('ханты'), '-', normalized('мансийский')
).interpretation(
    Region.name
)

BURAT = rule(
    caseless('усть'), '-', normalized('ордынский'),
    normalized('бурятский')
).interpretation(
    Region.name
)

AUTO_OKRUG = or_(
    rule(AUTO_OKRUG_NAME, AUTO_OKRUG_WORDS),
    or_(
        rule(
            HANTI,
            AUTO_OKRUG_WORDS,
            '-', normalized('югра')
        ),
        rule(
            caseless('хмао'),
        ).interpretation(Region.name),
        rule(
            caseless('хмао'),
            '-', caseless('югра')
        ).interpretation(Region.name),
    ),
    rule(
        BURAT,
        AUTO_OKRUG_WORDS
    )
).interpretation(
    Region
)


##########
#
#  RAION
#
###########


RAION_WORDS = or_(
    rule(caseless('р'), '-', in_caseless({'он', 'н'})),
    rule(normalized('район'))
).interpretation(
    Raion.type.const('район')
)

RAION_SIMPLE_NAME = and_(
    ADJF,
    TITLE
)

RAION_MODIFIERS = rule(
    in_caseless({
        'усть',
        'северо',
        'александрово',
        'гаврилово',
    }),
    DASH.optional(),
    TITLE
)

RAION_COMPLEX_NAME = rule(
    RAION_MODIFIERS,
    RAION_SIMPLE_NAME
)

RAION_NAME = or_(
    rule(RAION_SIMPLE_NAME),
    RAION_COMPLEX_NAME
).interpretation(
    Raion.name
)

RAION = rule(
    RAION_NAME,
    RAION_WORDS
).interpretation(
    Raion
)


###########
#
#   GOROD
#
###########


# Top 200 Russia cities, cover 75% of population

COMPLEX = morph_pipeline([
    'санкт-петербург',
    'нижний новгород',
    'н.новгород',
    'ростов-на-дону',
    'набережные челны',
    'улан-удэ',
    'нижний тагил',
    'комсомольск-на-амуре',
    'йошкар-ола',
    'старый оскол',
    'великий новгород',
    'южно-сахалинск',
    'петропавловск-камчатский',
    'каменск-уральский',
    'орехово-зуево',
    'сергиев посад',
    'новый уренгой',
    'ленинск-кузнецкий',
    'великие луки',
    'каменск-шахтинский',
    'усть-илимск',
    'усолье-сибирский',
    'кирово-чепецк',
])

SIMPLE = dictionary({
    'москва',
    'новосибирск',
    'екатеринбург',
    'казань',
    'самара',
    'омск',
    'челябинск',
    'уфа',
    'волгоград',
    'пермь',
    'красноярск',
    'воронеж',
    'саратов',
    'краснодар',
    'тольятти',
    'барнаул',
    'ижевск',
    'ульяновск',
    'владивосток',
    'ярославль',
    'иркутск',
    'тюмень',
    'махачкала',
    'хабаровск',
    'оренбург',
    'новокузнецк',
    'кемерово',
    'рязань',
    'томск',
    'астрахань',
    'пенза',
    'липецк',
    'тула',
    'киров',
    'чебоксары',
    'калининград',
    'брянск',
    'курск',
    'иваново',
    'магнитогорск',
    'тверь',
    'ставрополь',
    'симферополь',
    'белгород',
    'архангельск',
    'владимир',
    'севастополь',
    'сочи',
    'курган',
    'смоленск',
    'калуга',
    'чита',
    'орёл',
    'волжский',
    'череповец',
    'владикавказ',
    'мурманск',
    'сургут',
    'вологда',
    'саранск',
    'тамбов',
    'стерлитамак',
    'грозный',
    'якутск',
    'кострома',
    'петрозаводск',
    'таганрог',
    'нижневартовск',
    'братск',
    'новороссийск',
    'дзержинск',
    'шахта',
    'нальчик',
    'орск',
    'сыктывкар',
    'нижнекамск',
    'ангарск',
    'балашиха',
    'благовещенск',
    'прокопьевск',
    'химки',
    'псков',
    'бийск',
    'энгельс',
    'рыбинск',
    'балаково',
    'северодвинск',
    'армавир',
    'подольск',
    'королёв',
    'сызрань',
    'норильск',
    'златоуст',
    'мытищи',
    'люберцы',
    'волгодонск',
    'новочеркасск',
    'абакан',
    'находка',
    'уссурийск',
    'березники',
    'салават',
    'электросталь',
    'миасс',
    'первоуральск',
    'рубцовск',
    'альметьевск',
    'ковровый',
    'коломна',
    'керчь',
    'майкоп',
    'пятигорск',
    'одинцово',
    'копейск',
    'хасавюрт',
    'новомосковск',
    'кисловодск',
    'серпухов',
    'новочебоксарск',
    'нефтеюганск',
    'димитровград',
    'нефтекамск',
    'черкесск',
    'дербент',
    'камышин',
    'невинномысск',
    'красногорск',
    'мур',
    'батайск',
    'новошахтинск',
    'ноябрьск',
    'кызыл',
    'октябрьский',
    'ачинск',
    'северск',
    'новокуйбышевск',
    'елец',
    'евпатория',
    'арзамас',
    'обнинск',
    'каспийск',
    'элиста',
    'пушкино',
    'жуковский',
    'междуреченск',
    'сарапул',
    'ессентуки',
    'воткинск',
    'ногинск',
    'тобольск',
    'ухта',
    'серов',
    'бердск',
    'мичуринск',
    'киселёвск',
    'новотроицк',
    'зеленодольск',
    'соликамск',
    'раменский',
    'домодедово',
    'магадан',
    'глазов',
    'железногорск',
    'канск',
    'назрань',
    'гатчина',
    'саров',
    'новоуральск',
    'воскресенск',
    'долгопрудный',
    'бугульма',
    'кузнецк',
    'губкин',
    'кинешма',
    'ейск',
    'реутов',
    'железногорск',
    'чайковский',
    'азов',
    'бузулук',
    'озёрск',
    'балашов',
    'юрга',
    'кропоткин',
    'клин'
})

GOROD_ABBR = in_caseless({
    'спб',
    'мск',
    'нск'   # Новосибирск
})

GOROD_NAME = or_(
    rule(SIMPLE),
    COMPLEX,
    rule(GOROD_ABBR)
).interpretation(
    Settlement.name
)

SIMPLE = and_(
    TITLE,
    or_(
        NOUN,
        ADJF  # Железнодорожный, Юбилейный
    )
)

COMPLEX = or_(
    rule(
        SIMPLE,
        DASH.optional(),
        SIMPLE
    ),
    rule(
        TITLE,
        DASH.optional(),
        caseless('на'),
        DASH.optional(),
        TITLE
    )
)

NAME = or_(
    rule(SIMPLE),
    COMPLEX
)

MAYBE_GOROD_NAME = or_(
    NAME,
    rule(NAME, '-', INT)
).interpretation(
    Settlement.name
)

GOROD_WORDS = or_(
    rule(normalized('город')),
    rule(
        caseless('г'),
        DOT.optional()
    )
).interpretation(
    Settlement.type.const('город')
)

GOROD = or_(
    rule(GOROD_WORDS, MAYBE_GOROD_NAME),
    rule(
        GOROD_WORDS.optional(),
        GOROD_NAME
    )
).interpretation(
    Settlement
)


##########
#
#  SETTLEMENT NAME
#
##########


ADJS = gram('ADJS')
SIMPLE = and_(
    or_(
        NOUN,  # Александровка, Заречье, Горки
        ADJS,  # Кузнецово
        ADJF,  # Никольское, Новая, Марьино
    ),
    TITLE
)

COMPLEX = rule(
    SIMPLE,
    DASH.optional(),
    SIMPLE
)

NAME = or_(
    rule(SIMPLE),
    COMPLEX
)

SETTLEMENT_NAME = or_(
    NAME,
    rule(NAME, '-', INT),
    rule(NAME, ANUM)
)


###########
#
#   SELO
#
#############


SELO_WORDS = or_(
    rule(
        caseless('с'),
        DOT.optional()
    ),
    rule(normalized('село'))
).interpretation(
    Settlement.type.const('село')
)

SELO_NAME = SETTLEMENT_NAME.interpretation(
    Settlement.name
)

SELO = rule(
    SELO_WORDS,
    SELO_NAME
).interpretation(
    Settlement
)


###########
#
#   DEREVNYA
#
#############


DEREVNYA_WORDS = or_(
    rule(
        caseless('д'),
        DOT.optional()
    ),
    rule(normalized('деревня'))
).interpretation(
    Settlement.type.const('деревня')
)

DEREVNYA_NAME = SETTLEMENT_NAME.interpretation(
    Settlement.name
)

DEREVNYA = rule(
    DEREVNYA_WORDS,
    DEREVNYA_NAME
).interpretation(
    Settlement
)


###########
#
#   POSELOK
#
#############


POSELOK_WORDS = or_(
    rule(
        in_caseless({'п', 'пос'}),
        DOT.optional()
    ),
    rule(normalized('посёлок')),
    rule(
        caseless('р'),
        DOT.optional(),
        caseless('п'),
        DOT.optional()
    ),
    rule(
        normalized('рабочий'),
        normalized('посёлок')
    ),
    rule(
        caseless('пгт'),
        DOT.optional()
    ),
    rule(
        caseless('п'), DOT, caseless('г'), DOT, caseless('т'),
        DOT.optional()
    ),
    rule(
        normalized('посёлок'),
        normalized('городского'),
        normalized('типа'),
    ),
).interpretation(
    Settlement.type.const('посёлок')
)

POSELOK_NAME = SETTLEMENT_NAME.interpretation(
    Settlement.name
)

POSELOK = rule(
    POSELOK_WORDS,
    POSELOK_NAME
).interpretation(
    Settlement
)


##############
#
#   ADDR PERSON
#
############


ABBR = and_(
    length_eq(1),
    is_title()
)

PART = and_(
    TITLE,
    or_(
        gram('Name'),
        gram('Surn')
    )
)

MAYBE_FIO = or_(
    rule(TITLE, PART),
    rule(PART, TITLE),
    rule(ABBR, '.', TITLE),
    rule(ABBR, '.', ABBR, '.', TITLE),
    rule(TITLE, ABBR, '.', ABBR, '.')
)

POSITION_WORDS_ = or_(
    rule(
        dictionary({
            'мичман',
            'геолог',
            'подводник',
            'краевед',
            'снайпер',
            'штурман',
            'бригадир',
            'учитель',
            'политрук',
            'военком',
            'ветеран',
            'историк',
            'пулемётчик',
            'авиаконструктор',
            'адмирал',
            'академик',
            'актер',
            'актриса',
            'архитектор',
            'атаман',
            'врач',
            'воевода',
            'генерал',
            'губернатор',
            'хирург',
            'декабрист',
            'разведчик',
            'граф',
            'десантник',
            'конструктор',
            'скульптор',
            'писатель',
            'поэт',
            'капитан',
            'князь',
            'комиссар',
            'композитор',
            'космонавт',
            'купец',
            'лейтенант',
            'лётчик',
            'майор',
            'маршал',
            'матрос',
            'подполковник',
            'полковник',
            'профессор',
            'сержант',
            'старшина',
            'танкист',
            'художник',
            'герой',
            'княгиня',
            'строитель',
            'дружинник',
            'диктор',
            'прапорщик',
            'артиллерист',
            'графиня',
            'большевик',
            'патриарх',
            'сварщик',
            'офицер',
            'рыбак',
            'брат',
        })
    ),
    rule(normalized('генерал'), normalized('армия')),
    rule(normalized('герой'), normalized('россия')),
    rule(
        normalized('герой'),
        normalized('российский'), normalized('федерация')),
    rule(
        normalized('герой'),
        normalized('советский'), normalized('союз')
    ),
)

ABBR_POSITION_WORDS = rule(
    in_caseless({
        'адм',
        'ак',
        'акад',
    }),
    DOT.optional()
)

POSITION_WORDS = or_(
    POSITION_WORDS_,
    ABBR_POSITION_WORDS
)

MAYBE_PERSON = or_(
    MAYBE_FIO,
    rule(POSITION_WORDS, MAYBE_FIO),
    rule(POSITION_WORDS, TITLE)
)


###########
#
#   IMENI
#
##########


IMENI_WORDS = or_(
    rule(
        caseless('им'),
        DOT.optional()
    ),
    rule(caseless('имени'))
)

IMENI = or_(
    rule(
        IMENI_WORDS.optional(),
        MAYBE_PERSON
    ),
    rule(
        IMENI_WORDS,
        TITLE
    )
)

##########
#
#   LET
#
##########


LET_WORDS = or_(
    rule(caseless('лет')),
    rule(
        DASH.optional(),
        caseless('летия')
    )
)

LET_NAME = in_caseless({
    'влксм',
    'ссср',
    'алтая',
    'башкирии',
    'бурятии',
    'дагестана',
    'калмыкии',
    'колхоза',
    'комсомола',
    'космонавтики',
    'москвы',
    'октября',
    'пионерии',
    'победы',
    'приморья',
    'района',
    'совхоза',
    'совхозу',
    'татарстана',
    'тувы',
    'удмуртии',
    'улуса',
    'хакасии',
    'целины',
    'чувашии',
    'якутии',
})

LET = rule(
    INT,
    LET_WORDS,
    LET_NAME
)


##########
#
#    ADDR DATE
#
#############


MONTH_WORDS = dictionary({
    'январь',
    'февраль',
    'март',
    'апрель',
    'май',
    'июнь',
    'июль',
    'август',
    'сентябрь',
    'октябрь',
    'ноябрь',
    'декабрь',
})

DAY = and_(
    INT,
    gte(1),
    lte(31)
)

YEAR = and_(
    INT,
    gte(1),
    lte(2100)
)

YEAR_WORDS = normalized('год')

DATE = or_(
    rule(DAY, MONTH_WORDS),
    rule(YEAR, YEAR_WORDS)
)


#########
#
#   MODIFIER
#
############


MODIFIER_WORDS_ = rule(
    dictionary({
        'большой',
        'малый',
        'средний',

        'верхний',
        'центральный',
        'нижний',
        'северный',
        'дальний',

        'первый',
        'второй',

        'старый',
        'новый',

        'красный',
        'лесной',
        'тихий',
    }),
    DASH.optional()
)

ABBR_MODIFIER_WORDS = rule(
    in_caseless({
        'б', 'м', 'н'
    }),
    DOT.optional()
)

SHORT_MODIFIER_WORDS = rule(
    in_caseless({
        'больше',
        'мало',
        'средне',

        'верх',
        'верхне',
        'центрально',
        'нижне',
        'северо',
        'дальне',
        'восточно',
        'западно',

        'перво',
        'второ',

        'старо',
        'ново',

        'красно',
        'тихо',
        'горно',
    }),
    DASH.optional()
)

MODIFIER_WORDS = or_(
    MODIFIER_WORDS_,
    ABBR_MODIFIER_WORDS,
    SHORT_MODIFIER_WORDS,
)


##########
#
#   ADDR NAME
#
##########


ROD = gram('gent')

SIMPLE = and_(
    or_(
        ADJF,  # Школьная
        and_(NOUN, ROD),  # Ленина, Победы
    ),
    TITLE
)

COMPLEX = or_(
    rule(
        and_(ADJF, TITLE),
        NOUN
    ),
    rule(
        TITLE,
        DASH.optional(),
        TITLE
    ),
)

# TODO
EXCEPTION = dictionary({
    'арбат',
    'варварка'
})

MAYBE_NAME = or_(
    rule(SIMPLE),
    COMPLEX,
    rule(EXCEPTION)
)

NAME = or_(
    MAYBE_NAME,
    LET,
    DATE,
    IMENI
)

NAME = rule(
    MODIFIER_WORDS.optional(),
    NAME
)

ADDR_CRF = tag('I').repeatable()

NAME = or_(
    NAME,
    ANUM,
    rule(NAME, ANUM),
    rule(ANUM, NAME),
    rule(INT, DASH.optional(), NAME),
    rule(NAME, DASH, INT),
    ADDR_CRF
)

ADDR_NAME = NAME


########
#
#    STREET
#
#########


STREET_WORDS = or_(
    rule(normalized('улица')),
    rule(
        caseless('ул'),
        DOT.optional()
    )
).interpretation(
    Street.type.const('улица')
)

STREET_NAME = ADDR_NAME.interpretation(
    Street.name
)

STREET = or_(
    rule(STREET_WORDS, STREET_NAME),
    rule(STREET_NAME, STREET_WORDS)
).interpretation(
    Street
)


##########
#
#    PROSPEKT
#
##########


PROSPEKT_WORDS = or_(
    rule(
        in_caseless({'пр', 'просп'}),
        DOT.optional()
    ),
    rule(
        caseless('пр'),
        '-',
        in_caseless({'кт', 'т'}),
        DOT.optional()
    ),
    rule(normalized('проспект'))
).interpretation(
    Street.type.const('проспект')
)

PROSPEKT_NAME = ADDR_NAME.interpretation(
    Street.name
)

PROSPEKT = or_(
    rule(PROSPEKT_WORDS, PROSPEKT_NAME),
    rule(PROSPEKT_NAME, PROSPEKT_WORDS)
).interpretation(
    Street
)


############
#
#    PROEZD
#
#############


PROEZD_WORDS = or_(
    rule(caseless('пр'), DOT.optional()),
    rule(
        caseless('пр'),
        '-',
        in_caseless({'зд', 'д'}),
        DOT.optional()
    ),
    rule(normalized('проезд'))
).interpretation(
    Street.type.const('проезд')
)

PROEZD_NAME = ADDR_NAME.interpretation(
    Street.name
)

PROEZD = or_(
    rule(PROEZD_WORDS, PROEZD_NAME),
    rule(PROEZD_NAME, PROEZD_WORDS)
).interpretation(
    Street
)


###########
#
#   PEREULOK
#
##############


PEREULOK_WORDS = or_(
    rule(
        caseless('п'),
        DOT
    ),
    rule(
        caseless('пер'),
        DOT.optional()
    ),
    rule(normalized('переулок'))
).interpretation(
    Street.type.const('переулок')
)

PEREULOK_NAME = ADDR_NAME.interpretation(
    Street.name
)

PEREULOK = or_(
    rule(PEREULOK_WORDS, PEREULOK_NAME),
    rule(PEREULOK_NAME, PEREULOK_WORDS)
).interpretation(
    Street
)


########
#
#  PLOSHAD
#
##########


PLOSHAD_WORDS = or_(
    rule(
        caseless('пл'),
        DOT.optional()
    ),
    rule(normalized('площадь'))
).interpretation(
    Street.type.const('площадь')
)

PLOSHAD_NAME = ADDR_NAME.interpretation(
    Street.name
)

PLOSHAD = or_(
    rule(PLOSHAD_WORDS, PLOSHAD_NAME),
    rule(PLOSHAD_NAME, PLOSHAD_WORDS)
).interpretation(
    Street
)


############
#
#   SHOSSE
#
###########


# TODO
# Покровское 17 км.
# Сергеляхское 13 км
# Сергеляхское 14 км.


SHOSSE_WORDS = or_(
    rule(
        caseless('ш'),
        DOT
    ),
    rule(normalized('шоссе'))
).interpretation(
    Street.type.const('шоссе')
)

SHOSSE_NAME = ADDR_NAME.interpretation(
    Street.name
)

SHOSSE = or_(
    rule(SHOSSE_WORDS, SHOSSE_NAME),
    rule(SHOSSE_NAME, SHOSSE_WORDS)
).interpretation(
    Street
)


########
#
#  NABEREG
#
##########


NABEREG_WORDS = or_(
    rule(
        caseless('наб'),
        DOT.optional()
    ),
    rule(normalized('набережная'))
).interpretation(
    Street.type.const('набережная')
)

NABEREG_NAME = ADDR_NAME.interpretation(
    Street.name
)

NABEREG = or_(
    rule(NABEREG_WORDS, NABEREG_NAME),
    rule(NABEREG_NAME, NABEREG_WORDS)
).interpretation(
    Street
)

########
#
#  SAD
#
##########


SAD_WORDS = or_(
    rule(
        caseless('са'),
        DOT.optional()
    ),
    rule(normalized('сад'))
).interpretation(
    Street.type.const('сад')
)

SAD_NAME = ADDR_NAME.interpretation(
    Street.name
)

SAD = or_(
    rule(SAD_WORDS, SAD_NAME),
    rule(SAD_NAME, SAD_WORDS)
).interpretation(
    Street
)

########
#
#  POLE
#
##########


POLE_WORDS = or_(
    rule(
        caseless('пол'),
        DOT.optional()
    ),
    rule(normalized('поле'))
).interpretation(
    Street.type.const('поле')
)

POLE_NAME = ADDR_NAME.interpretation(
    Street.name
)

POLE = or_(
    rule(POLE_WORDS, POLE_NAME),
    rule(POLE_NAME, POLE_WORDS)
).interpretation(
    Street
)

########
#
#  KLAD
#
##########


KLAD_WORDS = or_(
    rule(
        caseless('клад'),
        DOT.optional()
    ),
    rule(normalized('кладбище'))
).interpretation(
    Street.type.const('кладбище')
)

KLAD_NAME = ADDR_NAME.interpretation(
    Street.name
)

KLAD = or_(
    rule(KLAD_WORDS, KLAD_NAME),
    rule(KLAD_NAME, KLAD_WORDS)
).interpretation(
    Street
)

########
#
#  PLAJ
#
##########


PLAJ_WORDS = or_(
    rule(
        caseless('пля'),
        DOT.optional()
    ),
    rule(normalized('пляж'))
).interpretation(
    Street.type.const('пляж')
)

PLAJ_NAME = ADDR_NAME.interpretation(
    Street.name
)

PLAJ = or_(
    rule(PLAJ_WORDS, PLAJ_NAME),
    rule(PLAJ_NAME, PLAJ_WORDS)
).interpretation(
    Street
)

########
#
#  PRUD
#
##########


PRUD_WORDS = or_(
    rule(
        caseless('пру'),
        DOT.optional()
    ),
    rule(normalized('пруд'))
).interpretation(
    Street.type.const('пруд')
)

PRUD_NAME = ADDR_NAME.interpretation(
    Street.name
)

PRUD = or_(
    rule(PRUD_WORDS, PRUD_NAME),
    rule(PRUD_NAME, PRUD_WORDS)
).interpretation(
    Street
)

########
#
#  VOKZAL
#
##########


VOKZAL_WORDS = or_(
    rule(
        caseless('вок'),
        DOT.optional()
    ),
    rule(normalized('вокзал'))
).interpretation(
    Street.type.const('вокзал')
)

VOKZAL_NAME = ADDR_NAME.interpretation(
    Street.name
)

VOKZAL = or_(
    rule(VOKZAL_WORDS, VOKZAL_NAME),
    rule(VOKZAL_NAME, VOKZAL_WORDS)
).interpretation(
    Street
)

########
#
#  METRO
#
##########


METRO_WORDS = or_(
    rule(
        caseless('ст'),
        DOT.optional()
    ),
    rule(normalized('метро'))
).interpretation(
    Street.type.const('метро')
)

METRO_NAME = ADDR_NAME.interpretation(
    Street.name
)

METRO = or_(
    rule(METRO_WORDS, METRO_NAME),
    rule(METRO_NAME, METRO_WORDS)
).interpretation(
    Street
)

########
#
#  TEATR
#
##########


TEATR_WORDS = or_(
    rule(
        caseless('т'),
        DOT.optional()
    ),
    rule(normalized('театр'))
).interpretation(
    Street.type.const('театр')
)

TEATR_NAME = ADDR_NAME.interpretation(
    Street.name
)

TEATR = or_(
    rule(TEATR_WORDS, TEATR_NAME),
    rule(TEATR_NAME, TEATR_WORDS)
).interpretation(
    Street
)

########
#
#  MUZEI
#
##########


MUZEI_WORDS = or_(
    rule(
        caseless('муз'),
        DOT.optional()
    ),
    rule(normalized('музей'))
).interpretation(
    Street.type.const('музей')
)

MUZEI_NAME = ADDR_NAME.interpretation(
    Street.name
)

MUZEI = or_(
    rule(MUZEI_WORDS, MUZEI_NAME),
    rule(MUZEI_NAME, MUZEI_WORDS)
).interpretation(
    Street
)

########
#
#  PAMETNIK
#
##########


PAMETNIK_WORDS = or_(
    rule(
        caseless('пам'),
        DOT.optional()
    ),
    rule(normalized('памятник'))
).interpretation(
    Street.type.const('памятник')
)

PAMETNIK_NAME = ADDR_NAME.interpretation(
    Street.name
)

PAMETNIK = or_(
    rule(PAMETNIK_WORDS, PAMETNIK_NAME),
    rule(PAMETNIK_NAME, PAMETNIK_WORDS)
).interpretation(
    Street
)

########
#
#  STATUIA
#
##########


STATUIA_WORDS = or_(
    rule(
        caseless('стат'),
        DOT.optional()
    ),
    rule(normalized('статуя'))
).interpretation(
    Street.type.const('статуя')
)

STATUIA_NAME = ADDR_NAME.interpretation(
    Street.name
)

STATUIA = or_(
    rule(STATUIA_WORDS, STATUIA_NAME),
    rule(STATUIA_NAME, STATUIA_WORDS)
).interpretation(
    Street
)

########
#
#  PARK
#
##########


PARK_WORDS = or_(
    rule(
        caseless('пар'),
        DOT.optional()
    ),
    rule(normalized('парк'))
).interpretation(
    Street.type.const('парк')
)

PARK_NAME = ADDR_NAME.interpretation(
    Street.name
)

PARK = or_(
    rule(PARK_WORDS, PARK_NAME),
    rule(PARK_NAME, PARK_WORDS)
).interpretation(
    Street
)

########
#
#  SQVER
#
##########


SQVER_WORDS = or_(
    rule(
        caseless('ск'),
        DOT.optional()
    ),
    rule(normalized('сквер'))
).interpretation(
    Street.type.const('сквер')
)

SQVER_NAME = ADDR_NAME.interpretation(
    Street.name
)

SQVER = or_(
    rule(SQVER_WORDS, SQVER_NAME),
    rule(SQVER_NAME, SQVER_WORDS)
).interpretation(
    Street
)

########
#
#  ROSCHA
#
##########


ROSCHA_WORDS = or_(
    rule(
        caseless('рощ'),
        DOT.optional()
    ),
    rule(normalized('роща'))
).interpretation(
    Street.type.const('роща')
)

ROSCHA_NAME = ADDR_NAME.interpretation(
    Street.name
)

ROSCHA = or_(
    rule(ROSCHA_WORDS, ROSCHA_NAME),
    rule(ROSCHA_NAME, ROSCHA_WORDS)
).interpretation(
    Street
)

########
#
#  BULVAR
#
##########


BULVAR_WORDS = or_(
    rule(
        caseless('б'),
        '-',
        caseless('р')
    ),
    rule(
        caseless('б'),
        DOT
    ),
    rule(
        caseless('бул'),
        DOT.optional()
    ),
    rule(normalized('бульвар'))
).interpretation(
    Street.type.const('бульвар')
)

BULVAR_NAME = ADDR_NAME.interpretation(
    Street.name
)

BULVAR = or_(
    rule(BULVAR_WORDS, BULVAR_NAME),
    rule(BULVAR_NAME, BULVAR_WORDS)
).interpretation(
    Street
)


##############
#
#   ADDR VALUE
#
#############


LETTER = in_caseless(set('абвгдежзиклмнопрстуфхшщэюя'))

QUOTE = in_(QUOTES)

LETTER = or_(
    rule(LETTER),
    rule(QUOTE, LETTER, QUOTE)
)

VALUE = rule(
    INT,
    LETTER.optional()
)

SEP = in_(r'/\-')

VALUE = or_(
    rule(VALUE),
    rule(VALUE, SEP, VALUE),
    rule(VALUE, SEP, LETTER)
)

ADDR_VALUE = rule(
    eq('№').optional(),
    VALUE
)


############
#
#    DOM
#
#############


DOM_WORDS = or_(
    rule(normalized('дом')),
    rule(
        caseless('д'),
        DOT
    )
).interpretation(
    Building.type.const('дом')
)

DOM_VALUE = ADDR_VALUE.interpretation(
    Building.number
)

DOM = rule(
    DOM_WORDS,
    DOM_VALUE
).interpretation(
    Building
)


###########
#
#  KORPUS
#
##########


KORPUS_WORDS = or_(
    rule(
        in_caseless({'корп', 'кор'}),
        DOT.optional()
    ),
    rule(normalized('корпус'))
).interpretation(
    Building.type.const('корпус')
)

KORPUS_VALUE = ADDR_VALUE.interpretation(
    Building.number
)

KORPUS = or_(
    rule(
        KORPUS_WORDS,
        KORPUS_VALUE
    ),
    rule(
        KORPUS_VALUE,
        KORPUS_WORDS
    )
).interpretation(
    Building
)


###########
#
#  STROENIE
#
##########


STROENIE_WORDS = or_(
    rule(
        caseless('стр'),
        DOT.optional()
    ),
    rule(normalized('строение'))
).interpretation(
    Building.type.const('строение')
)

STROENIE_VALUE = ADDR_VALUE.interpretation(
    Building.number
)

STROENIE = rule(
    STROENIE_WORDS,
    STROENIE_VALUE
).interpretation(
    Building
)


###########
#
#   OFIS
#
#############


OFIS_WORDS = or_(
    rule(
        caseless('оф'),
        DOT.optional()
    ),
    rule(normalized('офис'))
).interpretation(
    Room.type.const('офис')
)

OFIS_VALUE = ADDR_VALUE.interpretation(
    Room.number
)

OFIS = rule(
    OFIS_WORDS,
    OFIS_VALUE
).interpretation(
    Room
)


###########
#
#   KVARTIRA
#
#############


KVARTIRA_WORDS = or_(
    rule(
        caseless('кв'),
        DOT.optional()
    ),
    rule(normalized('квартира'))
).interpretation(
    Room.type.const('квартира')
)

KVARTIRA_VALUE = ADDR_VALUE.interpretation(
    Room.number
)

KVARTIRA = rule(
    KVARTIRA_WORDS,
    KVARTIRA_VALUE
).interpretation(
    Room
)


###########
#
#   INDEX
#
#############


INDEX = and_(
    INT,
    gte(100000),
    lte(999999)
).interpretation(
    Index.value
).interpretation(
    Index
)


#############
#
#   ADDR PART
#
############


ADDR_PART = or_(
    SAD,
    ROSCHA,
    SQVER,
    PARK,
    POLE,
    KLAD,
    PLAJ,
    PRUD,
    VOKZAL,
    METRO,
    TEATR,
    MUZEI,
    PAMETNIK
).interpretation(
    AddrPart.value
).interpretation(
    AddrPart
)



"""
This module contains the EmotionClassifiers class, which is designed to categorise input texts into emotion categories.
It uses a Huggingface transformer model trained on Bert_Large by default.
The EmotionClassifiers class has the following method:
@method:recognize_emotion: Adding an emotion category.
@method:recognize_average_emotion_from_multiple_models: Adding an average emotion category or the most likely emotion 
category using multiple models.
"""

from aniemore.recognizers.text import TextRecognizer
from aniemore.models import HuggingFaceModel
import torch
import pandas as pd
from tqdm import tqdm
import gc


class EmotionRecognizer:
    """
    This class is designed to categorize input texts into emotion categories.

    Attributes:

    - model: This attribute holds the model used for emotion recognition. It defaults to HuggingFaceModel.Text.Bert_Large,
      but can be set to any other compatible model during the instantiation of the class.

    - device: The device to use for inference. It automatically selects 'cuda' (GPU) if a compatible GPU
      is available and CUDA is enabled, otherwise, it falls back to 'cpu'.

    - text: The text to be analyzed.

    - df: The DataFrame containing the text to be analyzed.

    - text_column: The name of the column containing the text to be analyzed.
    """


    def __init__(self, model_name=HuggingFaceModel.Text.Bert_Tiny, device='cpu'):
        self.device = device
        self.model_name = model_name

        # Define the default model names to avoid repeated initialization
        self.default_model_names = [
            HuggingFaceModel.Text.Bert_Tiny,
            HuggingFaceModel.Text.Bert_Base,
            HuggingFaceModel.Text.Bert_Large,
            HuggingFaceModel.Text.Bert_Tiny2,
        ]

        self.recognizer = TextRecognizer(model=self.model_name, device=self.device)

    #def init_base_recognizer(self):
    #    self.recognizer = TextRecognizer(model=self.model_name, device=self.device)


    def recognize_emotion(self, text):
        """
        Return the emotion for a given text.
        """
        emotion = self.recognizer.recognize(text, return_single_label=True)
        return emotion

    def recognize_average_emotion_from_multiple_models(self, df, text_column, models=None, average=True):
        """
        Calculate the prevailing emotion using multiple models for a DataFrame column.
        """
        if models is None:
            models = self.default_model_names
        else:
            # Validate that the provided models are in the default models list
            for model in models:
                if model not in self.default_model_names:
                    raise ValueError(
                        f"Model {model} is not a valid model. Valid models are: {self.default_model_names}"
                    )

        # Initialize scores DataFrame
        scores = pd.DataFrame(
            0, index=df.index, columns=["happiness", "sadness", "anger", "fear", "disgust", "enthusiasm", "neutral"]
        )

        # Process each model one by one with progress bar
        for model_name in tqdm(models, desc="Processing models"):
            try:
                print(f"Clearing cache and collecting garbage before loading model: {model_name}")
                torch.cuda.empty_cache()
                gc.collect()

                print(f"Loading model: {model_name}")
                recognizer = TextRecognizer(model=model_name, device=self.device)
                model_results = [recognizer.recognize(text, return_single_label=False) for text in df[text_column]]

                for idx, result in enumerate(model_results):
                    for emotion, score in result.items():
                        if average:
                            scores.at[df.index[idx], emotion] += score
                        else:
                            scores.at[df.index[idx], emotion] = max(scores.at[df.index[idx], emotion], score)

                # Удаление модели из памяти
                del recognizer
                torch.cuda.empty_cache()  # Очистка кеша CUDA (если используется GPU)
                gc.collect()  # Сборка мусора
                print(f"Model {model_name} processed and unloaded.")
            except Exception as e:
                print(f"Error processing model {model_name}: {e}")
                torch.cuda.empty_cache()
                gc.collect()

        if average:
            # Average the scores by the number of models
            scores = scores.div(len(models))

        # Determine the prevailing emotion with the highest score
        prevailing_emotions = scores.idxmax(axis=1)

        return prevailing_emotions



import itertools

import networkx as nx
import geopy
import geopandas as gpd
import pandas as pd
from shapely.geometry import Point
from tqdm import tqdm


@staticmethod
def add_attributes(
    G: nx.classes.graph.Graph, new_attributes: dict, attribute_tag: str, toponym_attributes: bool
) -> nx.classes.graph.Graph:
    """
    Add attributes to nodes in the graph based on the specified conditions.

    Parameters:
        G (nx.classes.graph.Graph): The graph to which attributes will be added.
        new_attributes (dict): A dictionary containing the new attributes to be added.
        attribute_tag (str): The tag of the attribute to be added.
        toponym_attributes (bool): A boolean flag indicating whether to add attributes to toponyms.

    Returns:
        nx.classes.graph.Graph: The graph with the new attributes added.
    """

    if toponym_attributes:
        toponyms_list = [i for i in G.nodes if G.nodes[i].get("tag") == "TOPONYM"]
        for i in toponyms_list:
            G.nodes[i][attribute_tag] = new_attributes[i]

    else:
        word_list = [i for i in G.nodes if G.nodes[i].get("tag") != "TOPONYM"]
        for i in word_list:
            G.nodes[i][attribute_tag] = new_attributes[i]
    return G


@staticmethod
def add_city_graph(
    G: nx.classes.graph.Graph,
    districts: gpd.GeoDataFrame,
    municipals: gpd.GeoDataFrame,
    city_column: str,
    district_column: str,
    name_column: str,
    geometry_column: str,
    directed: bool = True,
) -> nx.classes.graph.Graph:
    """
    Add a city graph to the input graph based on the provided district and municipal data.

    Args:
        G (nx.classes.graph.Graph): The input graph.
        districts (gpd.GeoDataFrame): The district data.
        municipals (gpd.GeoDataFrame): The municipal data.
        city_column (str): The column name in districts containing the city data.
        district_column (str): The column name in municipals containing the district data.
        name_column (str): The column name in districts and municipals containing the name data.
        geometry_column (str): The column name in districts and municipals containing the geometry data.
        directed (bool): Whether the graph should be directed. Defaults to True.

    Returns:
        nx.classes.graph.Graph: The graph with the added city graph.
    """

    edges = []
    toponyms = [i for i in G.nodes if G.nodes[i]["tag"] == "TOPONYM"]
    city = districts[city_column].iloc[0]

    for i in range(len(districts)):
        name = districts[name_column].iloc[i]

        edges.append([city, name, "включает"])

    for i in range(len(municipals)):
        name = municipals[name_column].iloc[i]
        district = municipals[district_column].iloc[i]

        polygon = municipals[geometry_column].iloc[i]
        for j in toponyms:
            if "Lat" in G.nodes[j]:
                point = Point(G.nodes[j]["Lat"], G.nodes[j]["Lon"])

                if polygon.contains(point) or polygon.touches(point):
                    edges.append([name, j, "включает"])

        edges.append([district, name, "включает"])

    df = pd.DataFrame(edges, columns=["source", "target", "type"])

    if directed:
        city_graph = nx.from_pandas_edgelist(df, "source", "target", "type", create_using=nx.DiGraph)
    else:
        city_graph = nx.from_pandas_edgelist(df, "source", "target", "type")

    for i in range(len(districts)):
        if "population" in districts.columns:
            city_graph.nodes[districts[name_column].iloc[i]]["tag"] = "DISTRICT"
            city_graph.nodes[districts[name_column].iloc[i]]["population"] = districts[name_column].iloc[i]
        city_graph.nodes[districts[name_column].iloc[i]][geometry_column] = str(districts[geometry_column].iloc[i])

    for i in range(len(municipals)):
        if "population" in municipals.columns:
            city_graph.nodes[municipals[name_column].iloc[i]]["tag"] = "MUNICIPALITY"
            city_graph.nodes[municipals[name_column].iloc[i]]["population"] = municipals[name_column].iloc[i]
        city_graph.nodes[municipals[name_column].iloc[i]][geometry_column] = str(municipals[geometry_column].iloc[i])

    city_graph.nodes[city]["tag"] = "CITY"

    G = nx.compose(G, city_graph)

    return G


@staticmethod
def calculate_node_distances(
    G: nx.classes.graph.Graph, directed: bool = True
) -> nx.classes.graph.Graph or nx.classes.digraph.DiGraph:
    """
    Calculate the distances between pairs of nodes in the graph and add them as edges.

    Parameters:
        G (nx.classes.graph.Graph): The input graph.
        directed (bool): Whether the graph should be directed or undirected. Defaults to True.

    Returns:
        G: NetworkX graph object with added distance edges
    """

    toponyms = [i for i in G.nodes if G.nodes[i]["tag"] == "TOPONYM"]

    combinations = list(itertools.combinations(toponyms, 2))

    distance_edges = []

    for i in tqdm(combinations):
        if "Lat" in G.nodes[i[0]] and "Lat" in G.nodes[i[1]]:
            first_point = (G.nodes[i[0]]["Lat"], G.nodes[i[0]]["Lon"])
            second_point = (G.nodes[i[1]]["Lat"], G.nodes[i[1]]["Lon"])

            distance = geopy.distance.distance(first_point, second_point).km

            distance_edges.append([i[0], i[1], "удаленность", distance])
            distance_edges.append([i[1], i[0], "удаленность", distance])

    dist_edge_df = pd.DataFrame(distance_edges, columns=["source", "target", "type", "distance"])

    max_dist = dist_edge_df["distance"].max()
    for i in range(len(dist_edge_df)):
        dist_edge_df.at[i, "distance"] = dist_edge_df["distance"].iloc[i] / max_dist

    if directed:
        distance_graph = nx.from_pandas_edgelist(
            dist_edge_df, "source", "target", ["type", "distance"], create_using=nx.DiGraph
        )
    else:
        distance_graph = nx.from_pandas_edgelist(dist_edge_df, "source", "target", ["type", "distance"])

    G = nx.compose(G, distance_graph)

    return G



from loguru import logger
import sys

from .src import (
    EventDetection,
    Geocoder,
    TextClassifiers,
    GeoDataGetter,
    Semgraph,
    VKParser,
    City_services,
    AreaMatcher,
    EmotionRecognizer,
    RegionalActivity
)

__all__ = [
    "EventDetection",
    "TextClassifiers",
    "Geocoder",
    "GeoDataGetter",
    "Semgraph",
    "VKParser",
    "City_services",
    "AreaMatcher",
    "EmotionRecognizer",
    "RegionalActivity"
]

# logger.remove()
# logger.add(
#     sys.stdout,
#     format="<green>{time:MM-DD HH:mm}</green> | <level>{level: <8}</level> | <cyan>{message}</cyan>",
#     level="INFO",
#     colorize=True,
# )

import os
import sys

folder = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(folder)